@@ -16,7 +16,8 @@
 //! itself is simply a binding around these two crates to provide a
 //! convenience bridge between them. Go check them out! If you want
 //! more than a CLI, you can also use `jen` programmatically.
-#![doc(html_root_url = "https://docs.rs/jen/1.0.0")]
+#![allow(clippy::mutex_atomic)]
+#![doc(html_root_url = "https://docs.rs/jen/1.3.0")]
 use clap::{value_t, App, AppSettings, Arg};
 use serde_json::Value;
 
@@ -24,6 +25,7 @@ use jen::error::Error;
 use jen::generator::Generator;
 
 use std::io::{self, Write};
+use std::sync::{Arc, Mutex};
 use std::thread;
 
 /// Entry point of Jen.
@@ -42,7 +44,7 @@ fn run() -> Result<(), Error> {
     let args = build_cli(&core).get_matches();
 
     // unpack various arguments from the CLI to use later on
-    let amount = value_t!(args, "amount", usize).unwrap_or_else(|_| 1);
+    let limit = value_t!(args, "limit", usize).ok();
     let threads = value_t!(args, "workers", usize).unwrap_or_else(|_| cpus);
     let textual = args.is_present("textual");
     let template = args
@@ -50,61 +52,56 @@ fn run() -> Result<(), Error> {
         .expect("template argument should be provided")
         .to_owned();
 
-    // calculate options for threading
-    let total = amount / threads;
-    let offset = amount % threads;
+    // create a tracker used to keep counts
+    let tracker = Arc::new(Mutex::new(0));
 
-    // allocate vectors to store amount offsets
-    let mut amounts = Vec::with_capacity(threads);
+    // create all workers across multiple threads
+    let workers = (0..threads).map(move |_| {
+        // clone the state for ownership
+        let tracker = tracker.clone();
+        let template = template.clone();
 
-    // handle all remainders
-    for _ in 0..offset {
-        amounts.push(total + 1);
-    }
+        // spawn a thread to generate a batch
+        thread::spawn(move || -> Result<(), Error> {
+            // construct a new generator on the thread
+            let mut generator = Generator::from_path(template)?;
 
-    // push the total up to limit
-    while amounts.len() < threads {
-        amounts.push(total);
-    }
+            loop {
+                // fetch some amount of generated data from the generator
+                let created = generator.next().expect("unable to generate");
 
-    // spawn all workers
-    let workers = amounts
-        .into_iter()
-        .map(|amount| {
-            // clone the state for ownership
-            let template = template.clone();
-
-            // spawn a thread to generate a batch
-            thread::spawn(move || -> Result<(), Error> {
-                // construct a new generator on the thread
-                let generator = Generator::from_path(template)?;
-
-                // fetch some amount of generated data
-                for created in generator.take(amount) {
-                    // raw has no extras
-                    if textual {
-                        println!("{}", created);
-                        continue;
-                    }
-
-                    // compact the JSON to reduce any template based whitespace
-                    let parsed = serde_json::from_str::<Value>(&created)?;
-                    let output = serde_json::to_vec(&parsed)?;
-
-                    // lock stdout for writing
-                    let stdout = io::stdout();
-                    let mut stdout = stdout.lock();
-
-                    // write to stdout
-                    stdout.write_all(&output)?;
-                    stdout.write_all(b"\n")?;
+                // raw has no extras
+                if textual {
+                    println!("{}", created);
+                    continue;
                 }
 
-                // done!
-                Ok(())
-            })
+                // compact the JSON to reduce any template based whitespace
+                let parsed = serde_json::from_str::<Value>(&created)?;
+                let output = serde_json::to_vec(&parsed)?;
+
+                // lock stdout for writing
+                let stdout = io::stdout();
+                let mut stdout = stdout.lock();
+
+                // write to stdout
+                stdout.write_all(&output)?;
+                stdout.write_all(b"\n")?;
+
+                // lock the tracker for the time being
+                let mut counter = tracker.lock().unwrap();
+
+                // increment the counter
+                *counter += 1;
+
+                if let Some(limit) = limit {
+                    if *counter >= limit {
+                        return Ok(());
+                    }
+                }
+            }
         })
-        .collect::<Vec<_>>();
+    });
 
     // join all worker threads
     for worker in workers {
@@ -127,13 +124,12 @@ fn build_cli<'a, 'b>(cpus: &'a str) -> App<'a, 'b> {
         .version(env!("CARGO_PKG_VERSION"))
         // add the template arguments
         .args(&[
-            // amount: -a, --amount [1]
-            Arg::with_name("amount")
-                .help("The amount of documents to generate in this batch")
-                .short("a")
-                .long("amount")
-                .takes_value(true)
-                .default_value("1"),
+            // limit: -l, --limit [1]
+            Arg::with_name("limit")
+                .help("An upper limit of documents to generate")
+                .short("l")
+                .long("limit")
+                .takes_value(true),
             // textual: -t, --textual
             Arg::with_name("textual")
                 .help("Treat the input as textual, rather than JSON")
